//
//  VideoRecording.swift
//  SCNRecorder
//
//  Created by Vladislav Grigoryev on 11/03/2019.
//  Copyright © 2020 GORA Studio. https://gora.studio
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.

import Foundation
import AVFoundation

public final class VideoRecording {

    @Observable public var duration: TimeInterval = 0.0

    @Observable public var state: State = .preparing

    let videoOutput: VideoOutput

    public var url: URL { videoOutput.url }

    public var fileType: AVFileType { videoOutput.fileType }

    init(videoOutput: VideoOutput) { self.videoOutput = videoOutput }

    public func resume() { videoOutput.resume() }

    public func pause() { videoOutput.pause() }

    public func finish(completionHandler handler: @escaping (_ info: Info) -> Void) {
      videoOutput.finish { handler(Info(self)) }
    }

    func cancel() { videoOutput.cancel() }
}
